/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.

Contains directly, and indirectly, code from mpeg4ip, 
in particular the files mpeg4ip_bitstream.h and mpeg4ip.h are included.

Code from h264info V25, in particular the files addseidataDlg.cpp & .h, 
has been modified by stripping out most of the windows code and providing 
dummy functions for other parts.

The result is a linux cli binary called h264info. The only cli options I have tested
are "-iInputFile -oOutputFIle"

 */


/*
To compile in linux:
g++ -g -Wall -o h264fix h264fix.cpp  2>error
or for final production:
g++ -o h264fix h264fix.cpp

To debug:
gdb  h264fix
with gdb commands: run,backtrace,quit
*/

/* 
From addseidataDlg.cpp remove all #include, add the appropriate ones below 
Comment out CaddseidataDlg::OnLbnSelchangeList1, OnInitDialog 
Instead, use init (below) which is partially copied from onInitDialog.
Comment out OnTimer and all the On Button dialogs except CaddseidataDlg::OnButton1Click 
Comment out afxMessageBox but UNcomment some addLogLines if wanted.
Comment out AddLogLine and replace with versions below
Replace ftelli,fseeki with ftello,fseeko
As ftello returns off_t need to change from f_pos; Change all f_pos to __int64_t ?? 
Add .c_str() to buf_open invocations of CString.

From addseidataDlg.h remove all includes, add #include "mpeg4ip_bitstream.h" below 
Comment out all initial dialog stuff in class CaddseidataDlg down to the end of protected
Add simple new constructor: CaddseidataDlg(void) {} and bool init(int argc, char* __argv[]) ;  to class. 

Comment out all classes returning afx_msg except CaddseidataDlg::OnButton1Click
And OnTimer. Changed fpos_t to int64_t in buf_read

In both files use search/replace in editor to:
Replace "unsigned __int32" with uint32_t, etc
Replace "__int32" with __int32_t, etc

mpg4ip.h was missing so copied from V21
*/

using namespace std;

#include <cstdio>
#include <cmath>
#include <cstring>
#include <string>
#include <iostream>
#include "mpeg4ip_bitstream.h"

// Some replacement functions

#define CSBUFF_SIZE 2048
#define CSBUFF_SSIZE 1792  // leaves 256 for numeric formatting
static char CSbuffer [CSBUFF_SIZE]; 
class CString : public string {
	public:
		explicit CString() : string() {}
		CString(const char * instr) : string (instr)  {}
		CString(const string& instr) : string (instr)  {}
		//	Plus CString copy constructor is available by default	

// operators =, + and += are carried over from string class with auto type conversion
		bool operator== (const char * rstr) {if (compare(string(rstr)) == 0) return true; else return false;}
		bool operator== (CString& rstr) {if (compare(rstr) == 0) return true; else return false;}
		bool operator!= (const char * rstr) {if (compare(string(rstr)) == 0) return false; else return true;}
		bool operator!= (CString& rstr) {if (compare(rstr) == 0) return false; else return true;}

		CString Left(size_t len) {return CString(substr(0,len));}
		CString Right (size_t len) {if (len>size()) return CString(c_str()); else return CString(substr(size()-len, len)); } 
		const char * GetBuffer(void) {return c_str();} 				//could just use .c_str
		size_t	GetLength(void) {return size(); }						//could just use .size() or .length()
		unsigned int Atoi (void) {return atoi(c_str());} 

		void Format(const char * instr) {assign( instr);}
		void Format(CString& instr) {assign(instr);}

		void Format(const char * fstr, int num) {
			assign( CSbuffer, sprintf( CSbuffer, fstr, num)); 
			}
		void Format(const char * fstr, uint32_t& num) {
			assign( CSbuffer, sprintf(CSbuffer, fstr, num)); 
			}
		void Format(const char * fstr, uint32_t& num1, __int8_t& num2) {
			assign( CSbuffer, sprintf(CSbuffer, fstr, num1, num2)); 
			}
		void Format(const char * fstr, uint8_t& num1, int& num2) {
			assign( CSbuffer, sprintf(CSbuffer, fstr, num1, num2)); 
			}
		void Format(const char * fstr, unsigned int& num1, int& num2) {
			assign( CSbuffer, sprintf(CSbuffer, fstr, num1, num2)); 
			}
		void Format(const char * fstr, uint32_t& num1, uint32_t& num2) {
			assign( CSbuffer, sprintf(CSbuffer, fstr, num1, num2)); 
			}
		void Format(const char * fstr, uint32_t& num1, uint32_t& num2, uint32_t& num3) {
			assign( CSbuffer, sprintf(CSbuffer, fstr, num1, num2, num3)); }
		void Format(const char * fstr, int& num1, int& num2, __int64_t& num3) {
			assign( CSbuffer, sprintf(CSbuffer, fstr, num1, num2, num3)); }
		void Format(const char * fstr, unsigned int& num1, __int64_t& num2) {
			assign( CSbuffer, sprintf(CSbuffer, fstr, num1, num2)); }
		void Format(const char * fstr, int& num1, __int64_t& num2) {
			assign( CSbuffer, sprintf(CSbuffer, fstr, num1, num2) ); }
		// assume fstr + numbers will never be too large for CSbuffer, but instr may be too large
		void Format(const char * fstr, CString& instr, unsigned char& num) {
			if ( instr.size() > CSBUFF_SSIZE ) 	assign( "CString: Input string too large for CSbuffer"); 
			else 	 assign( CSbuffer, sprintf(CSbuffer, fstr, instr.c_str(), num));  
			}
		void Format(const char * fstr, uint32_t& num, CString& instr) {
			if ( instr.size() > CSBUFF_SSIZE ) { assign( "CString: Input string too large for CSbuffer"); }
			else 	{ assign( CSbuffer, sprintf(CSbuffer, fstr, num, instr.c_str())); } 
			}
		void Format(const char * fstr, CString& instr ) {	
			if ( instr.size() > CSBUFF_SSIZE ) { assign("CString: Input string too large for CSbuffer"); }
			else 	{ assign( CSbuffer, sprintf(CSbuffer, fstr, instr.c_str())); } }
	};

class CEdit : public string {
	public:
		explicit CEdit() : string() {}
		CEdit(const char * instr) : string (instr)  {}
		CEdit(const string& instr) : string (instr)  {}

		void SetWindowText(const char * instr) {assign(instr);}
		void GetWindowText(CString& instr) {instr.assign(c_str());}
	};

class CButton {
	bool pc ;
	public:
		explicit CButton() {pc = false;}
		void SetCheck(int ic ) {pc = (bool)ic;} //either 2 or 0 so far
		bool GetCheck( void ) {return pc;}
		void EnableWindow( int ic) {;} //dummy
	};

class CProgressCtrl {
	double lowpos,highpos;
	CString curpos;
	public:
	explicit CProgressCtrl() {;}
	void SetRange( double low, double high ) {lowpos=low; highpos=high;}
	void	SetPos(double pos) { curpos.Format("%d of 100",pos); cout <<'\xd'<< curpos.c_str()<<flush ;}
	};

class CListBox {
	public:
	explicit CListBox() {;}
	};

class CDialog {
	public:
	explicit CDialog() {;}
	void OnCancel() { exit(1);}
	};

#include "addseidataDlg.h"
#include "addseidataDlg.cpp"

//void CaddseidataDlg::AddLogLine( const char * LogText) { cout << LogText << endl ; }
//void CaddseidataDlg::AddLogLine( int level, const char * LogText) { cout << LogText << endl ; }
// void CaddseidataDlg::AddLogLine(  CString& LogText) { cout << LogText.c_str() << endl ; }
//void CaddseidataDlg::AddLogLine( int level,  CString& LogText) { cout << LogText.c_str() << endl ; }
int CaddseidataDlg::AddLogLine( int level,  CString LogText) { cout << LogText.c_str() << endl ; return 0;}

bool CaddseidataDlg::ProcessDialogMessages(void) {;}

bool CaddseidataDlg::init(int argc, char* __argv[]) {  // from old OnInitDialog()
	uint8_t exp_golomb_bits2[256] = {
		8, 7, 6, 6, 5, 5, 5, 5, 4, 4, 4, 4, 4, 4, 4, 4, 3, 
			3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 
			2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
			2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
			1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
			0, 
		};

	for( int ix=0; ix<256; ix++ ) {
		exp_golomb_bits[ix] = exp_golomb_bits2[ix];
		}

	bStopLoop = false;

	//set expgolomb bits for writes
	int ij = 1;
	int ix = 0;
	exp_golomb_bits_write[ix++] = ij;
	exp_golomb_bits_write[ix++] = ij++;	
	for( ; ix < 256; ) {
		for( int ik = 0; ik < pow((double)2, (double)(ij-1)) ; ik++ ) {
			exp_golomb_bits_write[ix++] = ij;
			}
		ij++;		
		}

	c_InputFileName.Format( "" );
	c_OutputFileName.Format( "" );
	c_PPS.SetCheck( 2 );

	deltaScale = new __int32_t [MAX_IX * MAX_IX];	
	RealPoc = new __int32_t [MAX_POC];

	lookaheadbuffer = (unsigned char*)malloc( 1500000 );
	slicebuffer     = (unsigned char*)malloc( 1500000 );
	payloadbuffer   = (unsigned char*)malloc( 32768 );
	sei_buffer      = (unsigned char*)malloc( 32768 );

	CString tempcstring, tempstring;
	CString whatpath, whatcommand;
	
	for (int i = 0; i<argc ; i++ ) {
		tempstring = __argv[i];		
		whatcommand = tempstring.Left(2);
		whatpath = tempstring.Right( tempstring.GetLength() - 2 ) ;

		if( whatcommand[1] == 'i' ) {			
			c_InputFile.SetWindowText( whatpath.GetBuffer() );
			automode = true;
			}
		if( whatcommand[1] == 'o' ) {
			c_OutputFile.SetWindowText( whatpath.GetBuffer() );
			automode = true;
			}
		if( whatcommand[1] == 'f' ) {
			c_FrameRate.SetWindowText( whatpath.GetBuffer() );
			automode = true;
			}
		if( whatcommand[1] == 'a' ) {
			c_AspectRatio.SetWindowText( whatpath.GetBuffer() );
			automode = true;
			}
		if( whatcommand[1] == 'l' ) {
			c_Level.SetWindowText( whatpath.GetBuffer() );
			automode = true;
			}
		if( whatcommand[1] == 'w' ) {
			c_Width.SetWindowText( whatpath.GetBuffer() );
			automode = true;
			}
		if( whatcommand[1] == 'h' ) {
			c_Height.SetWindowText( whatpath.GetBuffer() );
			automode = true;
			}
		if( whatcommand[1] == 'b' ) {
			c_MaxBitrate.SetWindowText( whatpath.GetBuffer() );
			automode = true;
			}
		if( whatcommand[1] == 'n' ) {
			c_NumRefFrames.SetWindowText( whatpath.GetBuffer() );
			automode = true;
			}
		if( whatcommand[1] == '3' ) {
			c_Pulldown.SetCheck( 2 );
			automode = true;
			}
		if( whatcommand[1] == 'r' ) {
			c_RemovePulldown.SetCheck( 2 );
			automode = true;
			}
		if( whatcommand[1] == '2' ) { //Hd 2 BR
			c_HDtoBR.SetCheck( 2 );
			automode = true;
			}
		if( whatcommand[1] == 'p' ) {
			c_AddPicStruct.SetCheck( 2 );
			automode = true;
			}
		if( whatcommand[1] == 'e' ) {
			c_PPS.SetCheck( 0 );
			automode = true;
			}
		}

	return true ; 
	}

int main(int argc, char* argv[]) {
	CaddseidataDlg temp;
	if (temp.init(argc, argv)) temp.OnBnClickedButton1();
	cout << endl ;
	return 0;
	
}


